﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [StorableClass]
  [Item("AfterCrossoverOperator", "A generic operator that can record genealogical relationships between crossover parents and children.")]
  public class BeforeCrossoverOperator<T> : EvolutionTrackingOperator, ICrossoverOperator<T> where T : class,IItem {
    private const string defaultParentsParameterName = "Parents";
    private const string defaultChildParameterName = "Child";

    public IScopeTreeLookupParameter<T> ParentsParameter { get; set; }
    public ILookupParameter<T> ChildParameter { get; set; }

    protected BeforeCrossoverOperator(BeforeCrossoverOperator<T> original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new BeforeCrossoverOperator<T>(this, cloner);
    }

    public BeforeCrossoverOperator() {
      ParentsParameter = new ScopeTreeLookupParameter<T>(defaultParentsParameterName);
      ChildParameter = new LookupParameter<T>(defaultChildParameterName);
      Parameters.Add(ParentsParameter);
      Parameters.Add(ChildParameter);
    }

    protected List<IGenealogyGraphNode<T>> CurrentGeneration {
      get {
        if (GenealogyGraph.Ranks.ContainsKey(Generations.Value))
          return GenealogyGraph.Ranks[Generations.Value].Cast<IGenealogyGraphNode<T>>().ToList();
        return null;
      }
    }

    public override IOperation Apply() {
      if (CurrentGeneration == null) throw new Exception();
      // we get the parents by parsing the scope name because the individuals in the execution scope are clones of the ones saved in the graph
      var parentVertices = ExecutionContext.Scope.SubScopes.Select(s => CurrentGeneration[int.Parse(s.Name)]).ToList();

      var parents = ParentsParameter.ActualValue.ToList();

      var childVertex = new GenealogyGraphNode<T> {
        // the child parameter does not have a value yet (it will be assigned after crossover), but the first parent is actually the future child so we use this
        Content = parents[0],
        Rank = parentVertices[0].Rank + 1
      };
      GenealogyGraph.AddVertex(childVertex);
      foreach (var v in parentVertices) {
        childVertex.AddReverseArc(v);
        v.AddForwardArc(childVertex);
      }

      return base.Apply();
    }
  }
}
