﻿using System.Collections.Generic;
using System.Drawing.Text;
using System.Globalization;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views;
using HeuristicLab.EvolutionTracking;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Visualization;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  [View("LineageExplorerView")]
  [Content(typeof(IGenealogyGraph), false)]
  public partial class SymbolicDataAnalysisExpressionLineageExplorerView : AsynchronousContentView {
    private readonly ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode> SymbolicExpressionTreeNodeLayoutEngine;
    private readonly SymbolicExpressionTreeLayoutAdapter SymbolicExpressionTreeLayoutAdapter;

    private readonly Dictionary<TreeNode, ISymbolicExpressionTree> TreeMap;

    private const int MinHorizontalSpacing = 50;
    private const int MinVerticalSpacing = 50;
    private const int PreferredNodeWidth = 80;
    private const int PreferredNodeHeight = 40;

    public new IGenealogyGraph Content {
      get { return (IGenealogyGraph)base.Content; }
      set { base.Content = value; }
    }

    public SymbolicDataAnalysisExpressionLineageExplorerView() {
      InitializeComponent();
      SymbolicExpressionTreeNodeLayoutEngine = new ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode> {
        MinHorizontalSpacing = MinHorizontalSpacing + PreferredNodeWidth,
        MinVerticalSpacing = MinVerticalSpacing + PreferredNodeHeight
      };
      SymbolicExpressionTreeLayoutAdapter = new SymbolicExpressionTreeLayoutAdapter();
      TreeMap = new Dictionary<TreeNode, ISymbolicExpressionTree>();
      double width = symbolicDataAnalysisExpressionLineageExplorerChart.PreferredSize.Width;
      double height = symbolicDataAnalysisExpressionLineageExplorerChart.PreferredSize.Height;
      if (symbolicDataAnalysisExpressionLineageExplorerChart.Chart == null) {
        symbolicDataAnalysisExpressionLineageExplorerChart.Chart = new Chart(0, 0, width, height);
      }
    }


    #region events
    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content != null) { updateTreeView(); }
    }
    #endregion

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      treeView.Enabled = Content != null;
    }

    private void updateTreeView() {
      var lastGen = Content.Ranks.OrderBy(x => x.Key).Last().Value.ToList();
      TreeMap.Clear();
      symbolicDataAnalysisExpressionLineageExplorerChart.Clear();
      foreach (var g in lastGen) {
        var treeNode = new TreeNode(g.Quality.ToString(CultureInfo.InvariantCulture));
        treeView.Nodes.Add(treeNode);
        TreeMap.Add(treeNode, (ISymbolicExpressionTree)g.Content);
      }
    }

    private void treeView_AfterSelect(object sender, TreeViewEventArgs e) {
      symbolicDataAnalysisExpressionLineageExplorerChart.Clear();
      var selectedNode = treeView.SelectedNode;
      var symbolicExpressionTree = TreeMap[selectedNode];
      var tile = new SymbolicExpressionTreeTile(this.symbolicDataAnalysisExpressionLineageExplorerChart.Chart) {
        LayoutAdapter = SymbolicExpressionTreeLayoutAdapter,
        LayoutEngine = SymbolicExpressionTreeNodeLayoutEngine,
        PreferredNodeWidth = PreferredNodeWidth,
        PreferredNodeHeight = PreferredNodeHeight
      };
      tile.SymbolicExpressionTree = symbolicExpressionTree;
      symbolicDataAnalysisExpressionLineageExplorerChart.Add(tile);

      //      graphics.TranslateTransform(0, symbolicDataAnalysisExpressionLineageExplorerChart.Height);
      //      graphics.ScaleTransform(1f, -1f); // flip vertically (because the visualization plugin is upside-down
      //      graphics.RotateTransform(180F);
      var graphics = symbolicDataAnalysisExpressionLineageExplorerChart.CreateGraphics();
      graphics.ScaleTransform(1f, -1f); // flip vertically (because the visualization plugin is upside-down
      graphics.TextRenderingHint = TextRenderingHint.AntiAlias;
      symbolicDataAnalysisExpressionLineageExplorerChart.Draw(graphics);
    }
  }
}
